// Package imports:
import 'package:zego_express_engine/zego_express_engine.dart';

/// @nodoc
enum MediaType {
  pureAudio,
  video,
  unknown,
}

/// @nodoc
class MediaPlayResult {
  int errorCode;
  String message;

  MediaPlayResult({
    required this.errorCode,
    this.message = '',
  });
}

/// @nodoc
class MediaSeekToResult {
  int errorCode;
  String message;

  MediaSeekToResult({
    required this.errorCode,
    this.message = '',
  });
}

/// @nodoc
enum MediaPlayState {
  /// Not playing
  noPlay,

  /// not start yet
  loadReady,

  /// Playing
  playing,

  /// Pausing
  pausing,

  /// End of play
  playEnded
}

extension MediaPlayStateExtension on MediaPlayState {
  static MediaPlayState fromZego(ZegoMediaPlayerState zegoMediaPlayerState) {
    switch (zegoMediaPlayerState) {
      case ZegoMediaPlayerState.NoPlay:
        return MediaPlayState.noPlay;
      case ZegoMediaPlayerState.Playing:
        return MediaPlayState.playing;
      case ZegoMediaPlayerState.Pausing:
        return MediaPlayState.pausing;
      case ZegoMediaPlayerState.PlayEnded:
        return MediaPlayState.playEnded;
    }
  }
}

/// @nodoc
/// Media Infomration of media file.
///
/// Meida information such as video resolution of media file.
class MediaInfo {
  /// Video resolution width.
  int width;

  /// Video resolution height.
  int height;

  /// Video frame rate.
  int frameRate;

  MediaInfo({
    required this.width,
    required this.height,
    required this.frameRate,
  });

  /// Constructs a media player information object by default.
  MediaInfo.defaultInfo()
      : width = 0,
        height = 0,
        frameRate = 0;

  MediaInfo.fromZego(ZegoMediaPlayerMediaInfo? zegoMediaInfo)
      : this(
          width: zegoMediaInfo?.width ?? 0,
          height: zegoMediaInfo?.height ?? 0,
          frameRate: zegoMediaInfo?.frameRate ?? 0,
        );
}
